<?php
/**
 * @file admin.php
 *
 * @brief Main file of admin panel (router)
 * @author Sobak
 * @version 1.0
 */

/**
 * Buffering is necessary
 *
 * Start session
 */
ob_start();
session_start();

/**
 * Start debug. Total script time and queries counter
 *
 * Debug will show in theme's footer.php file
 */
$scriptTimeStart = microtime(true);
$queriesCounter = 0;

/**
 * Set used variables (anty-notice)
 */
$cfg = array();

define('INSIDE', true);

/**
 * Load necessary files
 * Check whether config.php exists
 */

if (file_exists('config.php')) {
    require_once 'config.php';
}

else {
    die ('Cannot load configuration!');
}

if (file_exists('modules/logger.class.php')) {
    require_once 'modules/logger.class.php';
}

else {
    die ('Cannot load logger class!');
}


if (file_exists('modules/functions.php')) {
    require_once 'modules/functions.php';
}

else {
    die ('Cannot load functions.php!');
}


/**
 * load_module() is now included
 * logger is needed to handle errors
 *
 * create array which loaded modules
 */
$loadedModules = array();
loadModule ('db');
loadModule('user');

/**
 * Check whether user is logged and whether is admin
 */
header('Content-Type: text/html; charset=utf-8');
if (!usIsLogged ()) die ('<b>Błąd:</b><br />Nie jesteś zalogowany.');


/**
 * Connect to database
 * and load admin panel functions
 */
dbConnect();
loadModule('admin');

$user = usGet('', '');
if ($user['level'] != 1 && $user['level'] != 2) {
    logger::add_event (3, __FILE__, __LINE__, 'User attempted to enter the administration panel');
    die ('<b>Błąd:</b><br />Nie masz uprawnień do oglądania tej strony');
}

/**
 * Get important variables
 */
define('ADMIN_THEME', getSetting('admin_theme'));


/**
 * Include header
 */
require_once 'admin/themes/'.ADMIN_THEME.'/header.php';

$mode = $_GET['m'];


/**
 * Admin panel modules
 */
$adminModules = array ('summary', 'menu', 'menu_add', 'menu_edit',
    'arts', 'art_add', 'art_edit',
    'art_categories', 'art_category_add', 'art_category_edit',
    'pages', 'page_add', 'page_edit',
    'users', 'user_edit', 'user_add',
    'comments', 'trash', 'home', 'updates', 'del', 'logs',
    'stat_pages', 'stat_page_add', 'stat_page_edit');

if ($mode == '') {
    require_once 'admin/summary.php';
}

else {
    /**
     * Check whether file exists and it can be loaded
     */
    if (file_exists("admin/$mode.php") and in_array($mode, $adminModules))
    require_once "admin/$mode.php";

    else
    die ("Podany moduł nie istnieje lub nie mógł zostać załadowany");
}

/**
 * Include footer
 */
require_once 'admin/themes/'.ADMIN_THEME.'/footer.php';

?>