<?php
class bbcode
{
 public $config = array('site_address' => 'capaciouscore.pl', 'bbcode_on' => true, 'force_uid' => false, 'length_uid' => 10, 'youtube_width' => 480, 'youtube_height' => 360, 'lang' => array('quote' => 'Cytat', 'wrote' => 'napisa(a)', 'code' => 'Kod'));

 public function encode($str, $uid = '')
 {
  $uid = (isset($uid) ? $uid : $this -> config['force_uid'] == true ? $this -> make_uid() : '0');

  $html_entities_match = array('#&(?!(\#[0-9]+;))#', '#<#', '#>#', '#"#');
  $html_entities_replace = array('&amp;', '&lt;', '&gt;', '&quot;');

  $str = trim($str);
  $str = preg_replace($html_entities_match, $html_entities_replace, $str);

  if($this -> config['bbcode_on'] == true)
  {
   $str = $this -> encode_first_pass($str, $uid);
  }

  if($uid != '0')
  {
   return array('str' => $str, 'uid' => $uid);
  }
  else
  {
   return $str;
  }
 }
 
 public function decode($str)
 {
  $str = $this -> encode($str); 
  $str = $this -> clean_uid($str); // removing uid from bbcode
  $str = $this -> unprepare_data($str);  	

  return $str;
 }
 
 public function parse($str, $uid = '0')
 {
  $str = str_replace("\n", "\n<br />\n", $str);

  return $this -> fix_data($this -> config['bbcode_on'] == true ? $this -> clean_uid($this -> encode_second_pass($str, $uid)) : $this -> clean_uid($str));
 }
 
 public function make_clickable($str)
 {
  $str = $this -> prepare($str);
  // pad it with a space so we can match things at the start of the 1st line.
  $str = ' '.$str;
  // matches an "xxxx://yyyy" URL at the start of a line, or after a space.
  // xxxx can only be alpha characters.
  // yyyy is anything up to the first space, newline, comma, double quote or <
  $patterns[] = "#(^|[\n ])([\w]+?://[\w\#$%&~/.\-;:=,?@\[\]+]*)#is";
  $replacements[] = "\\1<a href=\"\\2\" target=\"_blank\">\\2</a>";
  // matches a "www|ftp.xxxx.yyyy[/zzzz]" kinda lazy URL thing
  // Must contain at least 2 dots. xxxx contains either alphanum, or "-"
  // zzzz is optional.. will contain everything up to the first space, newline,
  // comma, double quote or <.
  $patterns[] = "#(^|[\n ])((www|ftp)\.[\w\#$%&~/.\-;:=,?@\[\]+]*)#is";
  $replacements[] = "\\1<a href=\"http://\\2\" target=\"_blank\">\\2</a>";
  // matches an email@domain type address at the start of a line, or after a space.
  // Note: Only the followed chars are valid; alphanums, "-", "_" and or ".".
  $patterns[] = "#(^|[\n ])([a-z0-9&\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)*[\w]+)#i";
  $replacements[] = "\\1<a href=\"mailto:\\2@\\3\">\\2@\\3</a>";

  return substr(preg_replace($patterns, $replacements, $str), 1);
 }
 
 public function undo_make_clickable($str)
 {
  $patterns[] = "#<!-- BBCode auto-link start --><a href=\"(.*?)\" target=\"_blank\">.*?</a><!-- BBCode auto-link end -->#i";
  $patterns[] = "#<!-- BBcode auto-mailto start --><a href=\"mailto:(.*?)\">.*?</a><!-- BBCode auto-mailto end -->#i";
  $replacements[] = "\\1";
  $replacements[] = "\\1";

  return preg_replace($patterns, $replacements, $str);
 }
 
 public function undo_htmlspecialchars($str)
 {
  $match = array('/&gt;/i', '/&lt;/i', '/&quot;/i', '/&amp;/i');
  $replace = array('>', '<', '\'', '&');

  return preg_replace($match, $replace, $str);
 }
 
 public function clean_uid($str)
 {
  return preg_replace('/\:(([a-z0-9]:)?)0/s', '', $str);
 }
 
 private function make_uid()
 {
  // Jeeli wersja PHP < 4.2.0 wtedy zachodzi potrzeba uycia generator liczb losowych
  // mt_srand() > srand()
  // mt_srand((double)microtime() * 1000000);
  // lub
  // mt_srand(floor(time() / 86400));

  $hash = sha1(time().microtime());

  return substr($hash, 4, ($this -> config['length_uid'] > (strlen($hash) - 4) ? 10 : $this -> config['length_uid']));
 }
 
 private function clean_html($tag)
 {
  if(empty($tag[0]))
  {
   return '';
  }

  $allowed_html_tags = preg_split('/, */', strtolower($this -> config['allow_html_tags']));
  $disallowed_attributes = '/^(?:style|on)/i';

  // Check if this is an end tag
  preg_match('/<[^\w\/]*\/[\W]*(\w+)/', $tag[0], $matches);
  
  if(sizeof($matches))
  {
   $one_tag = '</'.$matches[1].'>';
   
   return (in_array(strtolower($matches[1]), $allowed_html_tags) ? $one_tag : htmlspecialchars($one_tag));
  }
  
  // Check if this is an allowed tag
  if(in_array(strtolower($tag[1]), $allowed_html_tags))
  {
   $attributes = '';

   if(!empty($tag[2]))
   {
    preg_match_all('/[\W]*?(\w+)[\W]*?=[\W]*?(["\'])((?:(?!\2).)*)\2/', $tag[2], $test);

    for($i = 0, $tmp['how'] = sizeof($test[0]); $i < $tmp['how']; ++$i)
    {
     if(preg_match($disallowed_attributes, $test[1][$i]))
     {
      continue;
     }
     
     $attributes .= ' '.$test[1][$i].'='.$test[2][$i].str_replace(array('[', ']'), array('&#91;', '&#93;'), htmlspecialchars($test[3][$i])).$test[2][$i];
    }
   }
		
   if(in_array(strtolower($tag[1]), $allowed_html_tags))
   {
    return '<'.$tag[1].$attributes.'>';
   }
   else
   {
    return htmlspecialchars('<'.$tag[1].$attributes.'>');
   }
  }
  else
  {
   return htmlspecialchars('<'.$tag[1].'>');
  }
 }

 private function encode_first_pass($str, $uid)
 {
  // pad it with a space so we can distinguish between FALSE and matching the 1st char (index 0).
  // This is important; bbencode_quote(), bbencode_list(), and bbencode_code() all depend on it.
  $str = ' '.$str;

  // [code] and [/code] for posting code (HTML, PHP, C etc etc) in your posts.
  $str = $this -> encode_first_pass_pda($str, $uid, '[code]', '[/code]', '', true, '');

  // [quote] and [/quote] for posting replies with quote, or just for quoting stuff.
  $str = $this -> encode_first_pass_pda($str, $uid, '[quote]', '[/quote]', '', false, '');
  $str = $this -> encode_first_pass_pda($str, $uid, '/\[quote=\\\\&quot;(.*?)\\\\&quot;\]/is', '[/quote]', '', false, '', "[quote=\\\"\\1\\\":$uid]");

  // [list] and [list=x] for (un)ordered lists.
  $open_tag = array(); // Stupid init :-)
  $open_tag[0] = '[list]';

  // unordered..
  $str = $this -> encode_first_pass_pda($str, $uid, $open_tag, '[/list]', '[/list:u]', false, 'replace_listitems');

  $open_tag[0] = '[list=none]';
  $open_tag[1] = '[list=1]';
  $open_tag[2] = '[list=a]';
  $open_tag[3] = '[list=i]';
  $open_tag[4] = '[list=circle]';
  $open_tag[5] = '[list=disc]';
  $open_tag[6] = '[list=square]';
  $open_tag[7] = '[list=decimal]';
  $open_tag[8] = '[list=lower-alpha]';
  $open_tag[9] = '[list=lower-roman]';
  $open_tag[10] = '[list=upper-alpha]';
  $open_tag[11] = '[list=upper-roman]';
  
  // ordered.
  $str = $this -> encode_first_pass_pda($str, $uid, $open_tag, '[/list]', '[/list:o]',  false, 'replace_listitems');

  // [table] and [/table] for making tables.
  $patterns[] = "#\[table\](.*?)\[/table\]#si";
  $replacements[] = "[table:$uid]\\1[/table:$uid]";
  // [mrow] for making tables.
  $patterns[] = "#\[mrow\](.*?)#si";
  $replacements[] = "[mrow:$uid]\\1";
  // [row] for making tables.
  $patterns[] = "#\[row\](.*?)#si";
  $replacements[] = "[row:$uid]\\1";
  // [row color=] for making tables.
  $patterns[] = "#\[row color=(\#[0-9a-fA-F]{3}|\#[0-9a-fA-F]{6}|[a-z\-]+)\](.*?)#si";
  $replacements[] = "[row color=\\1:$uid]\\2";
  // [col] for making tables.
  $patterns[] = "#\[col\](.*?)#si";
  $replacements[] = "[col:$uid]\\1";
  // [col color=] for making tables.
  $patterns[] = "#\[col color=(\#[0-9a-fA-F]{3}|\#[0-9a-fA-F]{6}|[a-z\-]+)\](.*?)#si";
  $replacements[] = "[col color=\\1:$uid]\\2";

  // [p=(left|center|right|justify)] and [/p]
  $patterns[] = "#\[p=(left|center|right|justify){1,1}\](.*?)\[/p\]#si";
  $replacements[] = "[p=\\1:$uid]\\2[/p:$uid]";

  // [color] and [/color] for setting text color
  $patterns[] = "#\[color=(\#[0-9a-fA-F]{3}|\#[0-9a-fA-F]{6}|[a-z\-]+)\](.*?)\[/color\]#si";
  $replacements[] = "[color=\\1:$uid]\\2[/color:$uid]";

  // [size] and [/size] for setting text size
  $patterns[] = "#\[size=([1-2]?[0-9]?)\](.*?)\[/size\]#si";
  $replacements[] = "[size=\\1:$uid]\\2[/size:$uid]";
  
  // [b] and [/b] for bolding text.
  $patterns[] = "#\[b\](.*?)\[/b\]#si";
  $replacements[] = "[b:$uid]\\1[/b:$uid]";

  // [u] and [/u] for underlining text.
  $patterns[] = "#\[u\](.*?)\[/u\]#si";
  $replacements[] = "[u:$uid]\\1[/u:$uid]";

  // [i] and [/i] for italicizing text.
  $patterns[] = "#\[i\](.*?)\[/i\]#si";
  $replacements[] = "[i:$uid]\\1[/i:$uid]";
  
  // [tt] and [/tt]
  $patterns[] = "#\[tt\](.*?)\[/tt\]#si";
  $replacements[] = "[tt:$uid]\\1[/tt:$uid]";

  // [s] and [/s] for strike
  $patterns[] = "#\[s\](.*?)\[/s\]#si";
  $replacements[] = "[s:$uid]\\1[/s:$uid]";
  
  // [sup] and [/sup]
  $patterns[] = "#\[sup\](.*?)\[/sup\]#si";
  $replacements[] = "[sup:$uid]\\1[/sup:$uid]";
  
  // [sub] and [/sub]
  $patterns[] = "#\[sub\](.*?)\[/sub\]#si";
  $replacements[] = "[sub:$uid]\\1[/sub:$uid]";
  
  // [abbr] and [/abbr]
  $patterns[] = "#\[abbr=(.*?)\](.*?)\[/abbr\]#si";
  $replacements[] = "[abbr=\\1:$uid]\\2[/abbr:$uid]";
  
  // [anchor] and [/anchor]
  $patterns[] = "#\[anchor\](.*?)\[/anchor\]#si";
  $replacements[] = "[anchor:$uid]\\1[/anchor:$uid]";
  
  // [h=x] and [/h]
  $patterns[] = "#\[h=([1-6])\](.*?)\[/h\]#si";
  $replacements[] = "[h=\\1:$uid]\\2[/h:$uid]";
  
  // [youtube] and [/youtube]
  $patterns[] = "#\[youtube\](.*?)\[/youtube\]#si";
  $replacements[] = "[youtube:$uid]\\1[/youtube:$uid]";
  
  // [img]image_url_here[/img] code..
  $patterns[] = "#\[img\]((http|ftp|https|ftps)://)([^ \?&=\#\"\n\r\t<]*?(\.(jpg|jpeg|gif|png)))\[/img\]#sie";
  $replacements[] = "'[img:$uid]\\1' . str_replace(' ', '%20', '\\3') . '[/img:$uid]'";

  // Remove our padding from the string..
  return substr(preg_replace($patterns, $replacements, $str), 1);
 }

 private function encode_second_pass($str, $uid)
 {
  global $bbcode_tpl;

  $str = $this -> prepare($str);
  // pad it with a space so we can distinguish between FALSE and matching the 1st char (index 0).
  // This is important; bbencode_quote(), bbencode_list(), and bbencode_code() all depend on it.
  $str = ' '.$str;
  // First: If there isn't a '[' and a ']' in the message, don't bother.
  if (!(strpos($str, '[') && strpos($str, ']')))
  {
   // Remove padding, return.
   $str = substr($str, 1);
   
   return $str;
  }

  // Only load the templates ONCE..
  if(!defined('BBCODE_TPL_READY'))
  {
   // Load templates from file into array.
   $bbcode_tpl = array('ulist_open' => '<ul class="list">',
                       'ulist_close' => '</ul>',
                       'olist_open' => '<ol class="list list_\\1">',
                       'olist_close' => '</ol>',
                       'listitem_open' => '<li>',
                       'listitem_close' => '</li>',
                       'quote_username' => '<p class="quote"><cite>{USERNAME}</cite> {L_WROTE}:</p> <blockquote><p>', // '{QUOTE}</p></blockquote>',
                       'quote_open' => '<p class="quote">'.$this -> config['lang']['quote'].':</p><blockquote><p>',
                       'quote_close' => '</p></blockquote>',
                       'code_open' => '<b>{L_CODE}:</b><pre class="code">',
                       'code_close' => '</pre>',
                       'p' => '<p class="text_\\1">\\2</p>',
                       'b_open' => '<b>',
                       'b_close' => '</b>',
                       'u_open' => '<u>',
                       'u_close' => '</u>',
                       'i_open' => '<i>',
                       'i_close' => '</i>',
                       'tt_open' => '<tt>',
                       'tt_close' => '</tt>',
                       's_open' => '<strike>',
                       's_close' => '</strike>',
                       'sup_open' => '<sup>',
                       'sup_close' => '</sup>',
                       'sub_open' => '<sub>',
                       'sub_close' => '</sub>',
                       'em' => '<em><span>\\1</span></em>',
                       'abbr' => '<abbr title="\\1">\\2</abbr>',
                       'anchor' => '<a name="\\1"></a>',
                       'header' => '<h\\1>\\2</h\\1>',
                       'youtube' => '<object type="application/x-shockwave-flash" data="http://www.youtube.com/v/{YOUTUBEID}&amp;hl=en" width="'.$this -> config['youtube_width'].'" height="'.$this -> config['youtube_height'].'"><param name="movie" value="http://www.youtube.com/v/{YOUTUBEID}&amp;hl=en" /><param name="FlashVars" value="playerMode=embedded" /><param name="wmode" value="transparent" /></object><br />',
                       'color_open' => '<span style="color: {COLOR}">',
                       'color_close' => '</span>',
                       'table_open' => '<table>',
                       'table_close' => '</td></tr></table>',
                       'table_mainrow' => '<tr class="mainrow"><td>',
                       'table_row' => '</td></tr><tr><td>',
                       'table_row_color' => '</td></tr><tr bgcolor="{TABCOLOR}"><td>',
                       'table_newcol' => '</td><td>',
                       'table_col_color' => '</td><td bgcolor="{TABCCOLOR}">',
                       'size_open' => '<span style="font-size: {SIZE}px; line-height: normal">',
                       'size_close' => '</span>',
                       'img'=> '<img src="{URL}" alt="{ALT}" />',
                       'url' => '<a href="{URL}" class="postlink">{DESCRIPTION}</a>',
                       'url_blank' => '<a href="{URL}" target="_blank" class="postlink">{DESCRIPTION}</a>',
                       'email' => '<a href="mailto:{EMAIL}">{EMAIL}</a>');

   // prepare array for use in regexps.
   $bbcode_tpl = $this -> prepare_bbcode_template($bbcode_tpl);
  }
   //  var_dump($bbcode_tpl); exit;
  $preg_patterns = $preg_replacements = $str_patterns = $str_replacements = array(); // Stupid init :-)

  // [code] and [/code] for posting code (HTML, PHP, C etc etc) in your posts.
  $str = $this -> encode_second_pass_code($str, $uid, $bbcode_tpl);

  // [quote] and [/quote] for posting replies with quote, or just for quoting stuff.
  $str_patterns[] = "[quote:$uid]";
  $str_patterns[] = "[/quote:$uid]";
  $str_replacements[] = $bbcode_tpl['quote_open'];
  $str_replacements[] = $bbcode_tpl['quote_close'];

  // New one liner to deal with opening quotes with usernames...
  // replaces the two line version that I had here before..
  $preg_patterns[] = "#\[quote=\"(.*?)\":$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['quote_username'];

  // [p=(left|center|right|justify)] and [/p]
  $preg_patterns[] = "#\[p=(left|center|right|justify):$uid\](.*?)\[/p:$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['p'];

  // [list] and [list=x] for (un)ordered lists.
  // unordered lists
  $str_patterns[] = "[list:$uid]";
  $str_replacements[] = $bbcode_tpl['ulist_open'];
  // li tags
  $str_patterns[] = "[*:$uid]";
  $str_patterns[] = "[/*:$uid]";
  $str_replacements[] = $bbcode_tpl['listitem_open'];
  $str_replacements[] = $bbcode_tpl['listitem_close'];
  // ending tags
  $str_patterns[] = "[/list:u:$uid]";
  $str_patterns[] = "[/list:o:$uid]";
  $str_replacements[] = $bbcode_tpl['ulist_close'];
  $str_replacements[] = $bbcode_tpl['olist_close'];
  
  // Ordered lists
  $preg_patterns[] = "/\[list=(.*?):$uid\]/si";
  $preg_replacements[] = $bbcode_tpl['olist_open'];
  
  // [table] and [/table] for making tables.
  // beginning code [table]
  $str_patterns[] = "[table:$uid]";
  $str_replacements[] = $bbcode_tpl['table_open'];
  // mainrow tag [mrow]
  $str_patterns[] = "[mrow:$uid]";
  $str_replacements[] = $bbcode_tpl['table_mainrow'];
  // row tag [row]
  $str_patterns[] = "[row:$uid]";
  $str_replacements[] = $bbcode_tpl['table_row'];
  // row tag with color [row color=]
  $preg_patterns[] = "#\[row color=(\#[0-9a-fA-F]{3}|\#[0-9a-fA-F]{6}|[a-z]+):$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['table_row_color'];
  // col tag with color [col color=]
  $preg_patterns[] = "#\[col color=(\#[0-9a-fA-F]{3}|\#[0-9a-fA-F]{6}|[a-z]+):$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['table_col_color'];
  // column tag [col]
  $str_patterns[] = "[col:$uid]";
  $str_replacements[] = $bbcode_tpl['table_newcol'];
  // ending tags [/table]
  $str_patterns[] = "[/table:$uid]";
  $str_replacements[] = $bbcode_tpl['table_close'];
  
  // colours
  $str_patterns[] = "[/color:$uid]";
  $str_replacements[] = $bbcode_tpl['color_close'];
  $preg_patterns[] = "#\[color=(\#[0-9a-fA-F]{3}|\#[0-9a-fA-F]{6}|[a-z]+):$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['color_open'];

  // size
  $str_patterns[] = "[/size:$uid]";
  $str_replacements[] = $bbcode_tpl['size_close'];
  $preg_patterns[] = "#\[size=([1-2]?[0-9]):$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['size_open'];

  // [b] and [/b] for bolding text.
  $str_patterns[] = "[b:$uid]";
  $str_patterns[] = "[/b:$uid]";
  $str_replacements[] = $bbcode_tpl['b_open'];
  $str_replacements[] = $bbcode_tpl['b_close'];

  // [u] and [/u] for underlining text.
  $str_patterns[] = "[u:$uid]";
  $str_patterns[] = "[/u:$uid]";
  $str_replacements[] = $bbcode_tpl['u_open'];
  $str_replacements[] = $bbcode_tpl['u_close'];

  // [i] and [/i] for italicizing text.
  $str_patterns[] = "[i:$uid]";
  $str_patterns[] = "[/i:$uid]";
  $str_replacements[] = $bbcode_tpl['i_open'];
  $str_replacements[] = $bbcode_tpl['i_close'];

  // [tt] and [/tt]
  $str_patterns[] = "[tt:$uid]";
  $str_patterns[] = "[/tt:$uid]";
  $str_replacements[] = $bbcode_tpl['tt_open'];
  $str_replacements[] = $bbcode_tpl['tt_close'];

  // [s] and [/s] for strike
  $str_patterns[] = "[s:$uid]";
  $str_patterns[] = "[/s:$uid]";
  $str_replacements[] = $bbcode_tpl['s_open'];
  $str_replacements[] = $bbcode_tpl['s_close'];

  // [sup] and [/sup]
  $str_patterns[] = "[sup:$uid]";
  $str_patterns[] = "[/sup:$uid]";
  $str_replacements[] = $bbcode_tpl['sup_open'];
  $str_replacements[] = $bbcode_tpl['sup_close'];

  // [sub] and [/sub]
  $str_patterns[] = "[sub:$uid]";
  $str_patterns[] = "[/sub:$uid]";
  $str_replacements[] = $bbcode_tpl['sub_open'];
  $str_replacements[] = $bbcode_tpl['sub_close'];

  // [abbr=x] and [/abbr]
  $preg_patterns[] = "#\[abbr=&quot;(.*?)&quot;:$uid\](.*?)\[/abbr:$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['abbr'];
  
  // [anchor] and [/anchor]
  $preg_patterns[] = "#\[anchor:$uid\](.*?)\[/anchor:$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['anchor'];
  
  // [h=x] and [/h]
  $preg_patterns[] = "#\[h=([1-6]):$uid\](.*?)\[/h:$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['header'];

  // [youtube]YouTube URL[/youtube] code..
  $preg_patterns[] = "#\[youtube:$uid\]http://(?:www\.)?youtube.com/watch\?v=([0-9A-Za-z-_]{11})[^[]*\[/youtube:$uid\]#si";
  $preg_replacements[] = $bbcode_tpl['youtube'];
  
  // [img]image_url_here[/img] code..
  // This one gets first-passed...
  $preg_callback_pattern = "#\[img:$uid\](.*?)\[/img:$uid\]#si";
  $preg_callback_target = array('bbcode', 'create_img');

  // 1) [url]xxxx://www.xxx.com[/url] code
  // 2) [url]www.xxx.com[/url] code.. (no xxxx:// prefix).
  // 3) [url=xxxx://www.xxx.com]xxx[/url] code
  // 4) [url=www.xxx.com]xxx[/url] code.. (no xxxx:// prefix).
  $this -> config['site_address'] = str_replace('.', '\.', $this -> config['site_address']);

  $preg_patterns[] = "#\[url\]([\w]+?://([a-zA-Z0-9\-]+\.)*".$this -> config['site_address']."([\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\]+]+|\[(?!url=))*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url1'];
  
  $preg_patterns[] = "#\[url\](([a-zA-Z0-9\-]+\.)*".$this -> config['site_address']."([\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\]+]+|\[(?!url=))*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url2'];
  
  $preg_patterns[] = "#\[url=([\w]+?://([a-zA-Z0-9\-]+\.)*".$this -> config['site_address']."[\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\[\]+]*?)\]([^?\n\r\t].*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url3'];

  $preg_patterns[] = "#\[url=(([a-zA-Z0-9\-]+\.)*".$this -> config['site_address']."[\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\[\]+]*?)\]([^?\n\r\t].*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url4'];

  $preg_patterns[] = "#\[url\]([\w]+?://([a-zA-Z0-9\-]+\.)*([\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\]+]+|\[(?!url=))*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url1_blank'];

  $preg_patterns[] = "#\[url\](([a-zA-Z0-9\-]+\.)*([\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\]+]+|\[(?!url=))*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url2_blank'];

  $preg_patterns[] = "#\[url=([\w]+?://([a-zA-Z0-9\-]+\.)*[\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\[\]+]*?)\]([^?\n\r\t].*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url3_blank'];

  $preg_patterns[] = "#\[url=(([a-zA-Z0-9\-]+\.)*[\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\[\]+]*?)\]([^?\n\r\t].*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url4_blank'];
  
  $preg_patterns[] = "#\[url_blank\]([\w]+?://([a-zA-Z0-9\-]+\.)*([\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\]+]+|\[(?!url=))*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url1_blank'];

  $preg_patterns[] = "#\[url_blank\](([a-zA-Z0-9\-]+\.)*([\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\]+]+|\[(?!url=))*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url2_blank'];

  $preg_patterns[] = "#\[url_blank=([\w]+?://([a-zA-Z0-9\-]+\.)*[\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\[\]+]*?)\]([^?\n\r\t].*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url3_blank'];

  $preg_patterns[] = "#\[url_blank=(([a-zA-Z0-9\-]+\.)*[\w\#󶼿ʣӦ\(\)$%&~/.\-;:=,?@\[\]+]*?)\]([^?\n\r\t].*?)\[/url\]#si";
  $preg_replacements[] = $bbcode_tpl['url4_blank'];

  // [email]user@domain.tld[/email] code..
  $preg_patterns[] = "#\[email\]([a-z0-9&\-_.]+?@[\w\-]+\.([\w\-\.]+\.)?[\w]+)\[/email\]#si";
  $preg_replacements[] = $bbcode_tpl['email'];

  $str = str_replace($str_patterns, $str_replacements, $str);
  $str = preg_replace($preg_patterns, $preg_replacements, $str);
  $str = preg_replace_callback($preg_callback_pattern, $preg_callback_target, $str);
  
  // Remove our padding from the string..
  return substr($str, 1);
 }
 
 private function prepare_bbcode_template($bbcode_tpl)
 {
  $bbcode_tpl['olist_open'] = str_replace('{LIST_TYPE}', '\\1', $bbcode_tpl['olist_open']);
  
  $bbcode_tpl['color_open'] = str_replace('{COLOR}', '\\1', $bbcode_tpl['color_open']);
  
  $bbcode_tpl['table_row_color'] = str_replace('{TABCOLOR}', '\\1', $bbcode_tpl['table_row_color']);
  $bbcode_tpl['table_col_color'] = str_replace('{TABCCOLOR}', '\\1', $bbcode_tpl['table_col_color']);
  
  $bbcode_tpl['size_open'] = str_replace('{SIZE}', '\\1', $bbcode_tpl['size_open']);

  $bbcode_tpl['quote_open'] = str_replace('{L_WROTE}', $this -> config['lang']['quote'], $bbcode_tpl['quote_open']);
  $bbcode_tpl['quote_open'] = str_replace('{L_QUOTE}', $this -> config['lang']['quote'], $bbcode_tpl['quote_open']);

  $bbcode_tpl['quote_username'] = str_replace('{USERNAME}', '\\1', $bbcode_tpl['quote_username']);
  $bbcode_tpl['quote_username'] = str_replace('{L_WROTE}', $this -> config['lang']['wrote'], $bbcode_tpl['quote_username']);
  $bbcode_tpl['quote_username'] = str_replace('{QUOTE}', '\\2', $bbcode_tpl['quote_username']);

  $bbcode_tpl['code_open'] = str_replace('{L_CODE}', $this -> config['lang']['code'], $bbcode_tpl['code_open']);
  
  $bbcode_tpl['youtube'] = str_replace('{YOUTUBEID}', '\\1', $bbcode_tpl['youtube']);
  $bbcode_tpl['youtube'] = str_replace('{YOUTUBELINK}', $lang['youtube_link'], $bbcode_tpl['youtube']);

  $bbcode_tpl['img'] = str_replace('{URL}', '\\1', $bbcode_tpl['img']);
  $bbcode_tpl['img'] = str_replace('{ALT}', '\\3', $bbcode_tpl['img']);
  
  // We do URLs in several different ways..
  $bbcode_tpl['url1'] = str_replace('{URL}', '\\1', $bbcode_tpl['url']);
  $bbcode_tpl['url1'] = str_replace('{DESCRIPTION}', '\\1', $bbcode_tpl['url1']);
  $bbcode_tpl['url1_blank'] = str_replace('{URL}', '\\1', $bbcode_tpl['url_blank']);
  $bbcode_tpl['url1_blank'] = str_replace('{DESCRIPTION}', '\\1', $bbcode_tpl['url1_blank']);

  $bbcode_tpl['url2'] = str_replace('{URL}', 'http://\\1', $bbcode_tpl['url']);
  $bbcode_tpl['url2'] = str_replace('{DESCRIPTION}', '\\1', $bbcode_tpl['url2']);
  $bbcode_tpl['url2_blank'] = str_replace('{URL}', 'http://\\1', $bbcode_tpl['url_blank']);
  $bbcode_tpl['url2_blank'] = str_replace('{DESCRIPTION}', '\\1', $bbcode_tpl['url2_blank']);

  $bbcode_tpl['url3'] = str_replace('{URL}', '\\1', $bbcode_tpl['url']);
  $bbcode_tpl['url3'] = str_replace('{DESCRIPTION}', '\\3', $bbcode_tpl['url3']);
  $bbcode_tpl['url3_blank'] = str_replace('{URL}', '\\1', $bbcode_tpl['url_blank']);
  $bbcode_tpl['url3_blank'] = str_replace('{DESCRIPTION}', '\\3', $bbcode_tpl['url3_blank']);

  $bbcode_tpl['url4'] = str_replace('{URL}', 'http://\\1', $bbcode_tpl['url']);
  $bbcode_tpl['url4'] = str_replace('{DESCRIPTION}', '\\3', $bbcode_tpl['url4']);
  $bbcode_tpl['url4_blank'] = str_replace('{URL}', 'http://\\1', $bbcode_tpl['url_blank']);
  $bbcode_tpl['url4_blank'] = str_replace('{DESCRIPTION}', '\\3', $bbcode_tpl['url4_blank']);

  $bbcode_tpl['email'] = str_replace('{EMAIL}', '\\1', $bbcode_tpl['email']);

  define('BBCODE_TPL_READY', true);

  return $bbcode_tpl;
 }

 private function encode_second_pass_code($str, $uid, $bbcode_tpl)
 {
  $code_start_html = $bbcode_tpl['code_open'];
  $code_end_html =  $bbcode_tpl['code_close'];

  // First, do all the 1st-level matches. These need an htmlspecialchars() run,
  // so they have to be handled differently.
  $match_count = preg_match_all("#\[code:1:$uid\](.*?)\[/code:1:$uid\]#si", $str, $matches);

  for($i = 0; $i < $match_count; ++$i)
  {
   $before_replace = $matches[1][$i];
   $after_replace = $matches[1][$i];

   // Replace 2 spaces with "&nbsp; " so non-tabbed code indents without making huge long lines.
   $after_replace = str_replace("  ", "&nbsp; ", $after_replace);
   // Now replace 2 spaces with " &nbsp;" to catch odd #s of spaces.
   $after_replace = str_replace("  ", " &nbsp;", $after_replace);

   // Replace tabs with "&nbsp; &nbsp;" so tabbed code indents sorta right without making huge long lines.
   $after_replace = str_replace("\t", "&nbsp; &nbsp;", $after_replace);

   // Now replace space occurring at the beginning of a line
   $after_replace = preg_replace("/^ {1}/m", '&nbsp;', $after_replace);

   $str_to_match = "[code:1:$uid]" . $before_replace . "[/code:1:$uid]";

   $replacement = $code_start_html;

   $after_replace = str_replace("\n<br />\n", "\n", $after_replace);
   $after_replace = html_entity_decode($after_replace);
   // Rem hiding before coloring
   $after_replace = preg_replace("#\[rem\](.*?)\[/rem\]#is", '\\1', $after_replace);
   // Coloring
   $after_replace = highlight_string($after_replace, true);
   // Removing <code> and </code>
   $after_replace = preg_replace("#<code>(.*?)</code>#is", '\\1', $after_replace);
   // Colors rem
   $after_replace = preg_replace("#(.*?)#i", $bbcode_tpl['em'], $after_replace);
   // Removing <br />
   $after_replace = str_replace("<br />", "", $after_replace);
   
   $replacement = $code_start_html;
   $replacement .= $after_replace;
   $replacement .= $code_end_html;

   $str = str_replace($str_to_match, $replacement, $str);
  }

  // Now, do all the non-first-level matches. These are simple.
  $str = str_replace("[code:$uid]", $code_start_html, $str);
  $str = str_replace("[/code:$uid]", $code_end_html, $str);

  return $str;
 }
 
 private function encode_first_pass_pda($str, $uid, $open_tag, $close_tag, $close_tag_new, $mark_lowest_level, $func, $open_regexp_replace = false)
 {
  $open_tag_count = 0;

  if (!$close_tag_new || ($close_tag_new == ''))
  {
   $close_tag_new = $close_tag;
  }

  $close_tag_length = strlen($close_tag);
  $close_tag_new_length = strlen($close_tag_new);
  $uid_length = strlen($uid);

  $use_function_pointer = ($func && ($func != ''));

  $stack = array(); // Stupid init :-)

  if(is_array($open_tag))
  {
   if(0 == count($open_tag))
   {
    // No opening tags to match, so return.
    return $str;
   }

   $open_tag_count = count($open_tag);
  }
  else
  {
   // only one opening tag. make it into a 1-element array.
   $open_tag_temp = $open_tag;
   $open_tag = array();
   $open_tag[0] = $open_tag_temp;
   $open_tag_count = 1;
  }

  $open_is_regexp = false;

  if($open_regexp_replace)
  {
   $open_is_regexp = true;

   if(!is_array($open_regexp_replace))
   {
    $open_regexp_temp = $open_regexp_replace;
    $open_regexp_replace = array(); // Stupid init :-)
    $open_regexp_replace[0] = $open_regexp_temp;
   }
  }

  // Can remove it :-)
  if($mark_lowest_level && $open_is_regexp)
  {
   // it should be a unsupported operation for bbcode_first_pass_pda()
  }

  // Start at the 2nd char of the string, looking for opening tags.
  $curr_pos = 1;

  while($curr_pos && ($curr_pos < strlen($str)))
  {
   $curr_pos = strpos($str, '[', $curr_pos);

   // If not found, $curr_pos will be 0, and the loop will end.
   if($curr_pos)
   {
    // We found a [. It starts at $curr_pos.
    // check if it's a starting or ending tag.
    $found_start = false;
    $which_start_tag = "";
    $start_tag_index = -1;

    for($i = 0; $i < $open_tag_count; $i++)
    {
     // Grab everything until the first "]"...
     $possible_start = substr($str, $curr_pos, strpos($str, ']', $curr_pos + 1) - $curr_pos + 1);

     // We're going to try and catch usernames with "[' characters.
     if(preg_match('#\[quote=\\\&quot;#si', $possible_start, $match) && !preg_match('#\[quote=\\\&quot;(.*?)\\\&quot;\]#si', $possible_start))
     {
      // OK we are in a quote tag that probably contains a ] bracket.
      // Grab a bit more of the string to hopefully get all of it..
      if($close_pos = strpos($str, '&quot;]', $curr_pos + 14))
      {
       if(strpos(substr($str, $curr_pos + 14, $close_pos - ($curr_pos + 14)), '[quote') === false)
       {
       	$possible_start = substr($str, $curr_pos, $close_pos - $curr_pos + 7);
       }
      }
     }

     // Now compare, either using regexp or not.
     if($open_is_regexp)
     {
      $match_result = array(); // Stupid init :-)

      if(preg_match($open_tag[$i], $possible_start, $match_result))
      {
       $found_start = true;
       $which_start_tag = $match_result[0];
       $start_tag_index = $i;
       break;
      }
     }
     else
     {
      // straightforward string comparison.
      if(0 == strcasecmp($open_tag[$i], $possible_start))
      {
       $found_start = true;
       $which_start_tag = $open_tag[$i];
       $start_tag_index = $i;
       break;
      }
     }
    }
    
    if($found_start)
    {
     // We have an opening tag.
     // Push its position, the text we matched, and its index in the open_tag array on to the stack, and then keep going to the right.
     $match = array('pos' => $curr_pos, 'tag' => $which_start_tag, 'index' => $start_tag_index);
     array_push($stack, $match);
     // Rather than just increment $curr_pos
     // Set it to the ending of the tag we just found
     // Keeps error in nested tag from breaking out
     // of table structure..
     $curr_pos += strlen($possible_start);
    }
    else
    {
     // check for a closing tag..
     $possible_end = substr($str, $curr_pos, $close_tag_length);
     if(strcasecmp($close_tag, $possible_end) == 0)
     {
      // We have an ending tag.
      // Check if we've already found a matching starting tag.
      if(sizeof($stack) > 0)
      {
       // There exists a starting tag.
       $curr_nesting_depth = sizeof($stack);
       // We need to do 2 replacements now.
       $match = array_pop($stack);
       $start_index = $match['pos'];
       $start_tag = $match['tag'];
       $start_length = strlen($start_tag);
       $start_tag_index = $match['index'];
         					
       if($open_is_regexp)
       {
        $start_tag = preg_replace($open_tag[$start_tag_index], $open_regexp_replace[$start_tag_index], $start_tag);
       }

       // everything before the opening tag.
       $before_start_tag = substr($str, 0, $start_index);
       // everything after the opening tag, but before the closing tag.
       $between_tags = substr($str, $start_index + $start_length, $curr_pos - $start_index - $start_length);
       // Run the given function on the text between the tags..
       if($use_function_pointer)
       {    // @TODO: look at called function
        $between_tags = $this -> $func($between_tags, $uid);
       }
       
       // everything after the closing tag.
       $after_end_tag = substr($str, $curr_pos + $close_tag_length);
       // Mark the lowest nesting level if needed.
       if($mark_lowest_level && ($curr_nesting_depth == 1))
       {
        if($open_tag[0] == '[code]')
	{
	 $code_entities_match = array('#<#', '#>#', '#"#', '#:#', '#\[#', '#\]#', '#\(#', '#\)#', '#\{#', '#\}#');
	 $code_entities_replace = array('&lt;', '&gt;', '&quot;', '&#58;', '&#91;', '&#93;', '&#40;', '&#41;', '&#123;', '&#125;');
	 $between_tags = preg_replace($code_entities_match, $code_entities_replace, $between_tags);
	}
	
        $str = $before_start_tag . substr($start_tag, 0, $start_length - 1) . ":$curr_nesting_depth:$uid]";
	$str .= $between_tags . substr($close_tag_new, 0, $close_tag_new_length - 1) . ":$curr_nesting_depth:$uid]";
       }	
       else
       {
       	if($open_tag[0] == '[code]')
        {
	 $str = $before_start_tag . '&#91;code&#93;';
         $str .= $between_tags . '&#91;/code&#93;';
	}
        else
	{
	 if($open_is_regexp)
	 {
	  $str = $before_start_tag . $start_tag;
	 }
	 else
	 {
	  $str = $before_start_tag . substr($start_tag, 0, $start_length - 1) . ":$uid]";
	 }
	 
	 $str .= $between_tags . substr($close_tag_new, 0, $close_tag_new_length - 1) . ":$uid]";
	}
       }

       $str .= $after_end_tag;
       // Now.. we've screwed up the indices by changing the length of the string.
       // So, if there's anything in the stack, we want to resume searching just after it.
       // otherwise, we go back to the start.
       if(sizeof($stack) > 0)
       {
      	$match = array_pop($stack);
      	$curr_pos = $match['pos'];
        // bbcode_array_push($stack, $match);
        // ++$curr_pos;
       }
       else
       {
       	$curr_pos = 1;
       }
      }
      else
      {
       // No matching start tag found. Increment pos, keep going.
       ++$curr_pos;
      }
     }
     else
     {
      // No starting tag or ending tag.. Increment pos, keep looping.,
      ++$curr_pos;
     }
    }
   }
  }

  return $str;
 }

 private function replace_listitems($str, $uid)
 {
  return str_replace(array('[*]', '[/*]'), array('[*:'.$uid.']', '[/*:'.$uid.']'), $str);
 }

 private function unprepare_data($str)
 {
  $unhtml_specialchars_match = array('#&gt;#', '#&lt;#', '#&quot;#', '#&amp;#');
  $unhtml_specialchars_replace = array('>', '<', '"', '&');

  return preg_replace($unhtml_specialchars_match, $unhtml_specialchars_replace, $str);
 }
 
 private function prepare($str)
 {
  return preg_replace('#(script|about|applet|activex|chrome):#is', "\\1&#058;", $str);
 }
 
 private function create_img($matches)
 {
  global $bbcode_tpl;

  return preg_replace("#((http|https|ftp|ftps://)*+.*/(.+\.(jpg|jpeg|gif|png)*+))#is", $bbcode_tpl['img'], $matches[1]);
 }
 
 private function fix_data($str)
 {
  $str = preg_replace("#<br />(\n|\r\n)*?<li#si", "<li", $str);
  $str = preg_replace("#<br />(\n|\r\n)*?</li>#si", "</li>", $str);
  $str = preg_replace("#<br />(\n|\r\n)*?<ul>#si", "<ul>", $str);
  $str = preg_replace("#<br />(\n|\r\n)*?</ul>#si", "</ul>", $str);
  $str = preg_replace("#</ul>(\n|\r\n)*?<br />#si", "</ul>", $str);
  $str = preg_replace("#</ol>(\n|\r\n)*?<br />#si", "</ol>", $str);
  $str = preg_replace("#<br />(\n|\r\n)*?<ol#si", "<ol", $str);
  $str = preg_replace("#<br />(\n|\r\n)*?</ol>#si", "</ol>", $str);
  $str = preg_replace("#</h([1-6])>( )*(\r|\r\n)*?<br />#si", "</h\\1>", $str);
  $str = preg_replace("#</blockquote>(\n|\r\n)*?<br />#si", "</blockquote>", $str);
  $str = preg_replace("#</pre>(\n|\r\n)*?<br />#si", "</pre>", $str);
  $str = preg_replace("#</p>(\n|\r\n)*?<br />#si", "</p>", $str);
  $str = preg_replace("#<pre class=\"code\"><span style=\"color: \#000000\">\n#si", "<pre class=\"code\"><span style=\"color: #000000\">", $str);
  $str = preg_replace("#</span>\n</pre>#si", "</span></pre>", $str);
//  <b>Kod:</b><pre class="code"><span style="color: #000000">

//</span><span style="color: #0000BB">?&gt;</span>
//
//
  return $str;
 }
}
?>
